import logging

from tqdm import tqdm
from pygendata.ddl import DDL
from pygendata.managers import csvmanager, tsvmanager, jsonmanager
from pygendata.templates.geo import GeoTemplate
from pygendata.cli import run
from multiprocessing import Pool, cpu_count

# TODO: support reading from json, tsv
class DataGenerator:
    """
    DataGenerator takes a manager of type (csv, tsv, json ..ect)
    DataGenerator takes an optional argument for number of rows to generate when dealing with csv
    """
    def __init__(self, manager, **kwargs):
        if kwargs.get('rows'):
            self.rows = kwargs['rows']
        else:
            self.rows = []
        if manager == 'csv':
            self.manager = csvmanager.CSVManager()
        elif manager == 'tsv':
            self.manager = tsvmanager.TSVManager()
        elif manager == 'json':
            self.manager = jsonmanager.JSONManager()
    
    @property
    def manager(self):
        return self._manager
    
    @manager.setter
    def manager(self, manager):
        self._manager = manager

    def ddl(self, infile, outfile):
        """
        reads a ddl file from disk and generates a csv based on the column names
        """
        try:
            statement = self.manager.read(infile)
            ddl = DDL(statement)
            ddl.get_columns()
            ddl.create_headers()
            self.manager.headers = ddl.headers
            p = Pool(cpu_count())
            print('Generating rows from ddl file')
            results = p.map(ddl.create_row, tqdm(range(self.rows)))
            self.manager.rows = list(results)
            self.manager.write(outfile)
        except IOError as e:
            logging.warn(str(e))
    
    # TODO: template for high cardinality data (cardinality column name for cli)
    # TODO: make geo data realistic? see what kind of geo data would want to be generated
    def template(self, template_type, template_arg, outfile):
        """
        Generates data using a template file, eventually this should support custom templates, currently only supports geo
        """
        template = None
        try:
            if template_type == 'geo':
                template = GeoTemplate(template_arg)
            p = Pool(cpu_count())
            self.manager.headers = template.keys
            print('Generating rows from template')
            results = p.map(template.generate, tqdm(range(self.rows)))
            self.manager.rows = list(results)
            self.manager.write(outfile)
        except IOError as e:
            logging.warn(str(e))


if __name__ == "__main__":
    run()



from faker import Faker

fake = Faker()

# Point, Polygon, and Multipolygon currently not supported
datatypes = {
    'BIGINT': fake.pyint,
    'TIMESTAMP(0)': fake.date_time,
    'INTEGER': fake.pyint,
    'TEXTENCODINGDICT(32)': fake.pystr,
    'FLOAT': fake.pyfloat,
    'SMALLINT': fake.pyint,
    'BIGINTENCODINGFIXED(8)': fake.pyint,
    'BIGINTENCODINGFIXED(16)': fake.pyint,
    'BIGINTENCODINGFIXED(32)': fake.pyint,
    'BOOLEAN': True,
    'DATE[1]': fake.date,
    'DATEENCODINGFIXED(16)': fake.date_time,
    'DATEENCODINGFIXED(32)': fake.date_time,
    'DECIMAL': fake.pyfloat,
    'DOUBLE': fake.pyfloat,
    'EPOCH': fake.date_time,
    'FLOAT': fake.pyfloat,
    'INTEGERENCODINGFIXED(8)': fake.pyint,
    'INTEGERENCODINGFIXED(16)': fake.pyint,
    'SMALLINTENCODINGFIXED(8)': fake.pyint,
    'TEXTENCODINGDICT(8)': fake.pystr,
    'TEXTENCODINGDICT(16)': fake.pystr,
    'TEXTENCODINGNONE)': fake.pystr,
    'TIME': fake.date_time(),
    'TIMEENCODINGFIXED(32)': fake.date_time,
    'TIMESTAMP(3)': fake.date_time,
    'TIMESTAMP(6)': fake.date_time,
    'TIMESTAMP(9)': fake.date_time,
    'TIMESTAMPENCODINGFIXED(32)': fake.date_time,
    'TINYINT': fake.pyint
}

# we match on special types to make the data more realistic
special_types = {
    'EMAIL': fake.email,
    'NAME': fake.name,
}


from pygendata.datatypes import datatypes, special_types
from pygendata.exceptions import TypeNotSupportedError

import re

class DDL:
    def __init__(self, statement):
        self.statement = statement
        self.columns = []
        self.column_data = [] # {'name': <column_name>, 'type': <column_type> }
        self.headers = []

    @property
    def column_data(self):
        return self._column_data
    
    @column_data.setter
    def column_data(self, column_data):
        self._column_data = column_data
    
    def get_columns(self):
        _, *cols = self.statement.split('\n')
        cols = [x.rstrip(',') for x in cols]
        cols = [x for x in cols if x != ');'] # remove );
        cols = list(filter(None, cols)) # remove empty strings
        self.columns = cols
    
    def create_headers(self):
        for column in self.columns:
            name, *_ = column.split(' ')
            self.headers.append(name)

    def create_row(self, current_row):
        c = {}
        for column in self.columns:
            name, *type_info = column.split(' ')
            type_info = ''.join(type_info)
            if type_info in datatypes:
                # is name special?
                patterns = re.compile(r'(email)|(name)', re.IGNORECASE)
                matches = patterns.match(name)
                id_pattern = re.compile(r'(id)', re.IGNORECASE)
                id_matches = id_pattern.match(name)
                if matches:
                    c[name] = special_types[name.upper()]()
                elif id_matches:
                    c[name] = current_row
                else:
                    c[name] = datatypes[type_info]() # expensive
            else:
                raise TypeNotSupportedError(f"{c['type']} is not currently supported")
        return c


from faker import Faker

class GeoTemplate:
    """
    A geo template generates Lat/Lon values particular to a specified region
    """

    # TODO: support more regions
    _allowed_regions = {'US': True, 'GB': True}

    def __init__(self, region):
        self.fake = Faker()
        if self.allowed_region(region):
            self.region = region
        else:
            self.region = 'US' # defaults to US
        self.keys = ['latitude', 'longitude']
        self.values = []
    
    def allowed_region(self, region):
        if region not in self._allowed_regions:
            return False
        return True
    
    def generate(self, row):
        latitude, longitude = self.fake.local_latlng(country_code=self.region, coords_only=True)
        return { 'latitude': latitude, 'longitude': longitude }
            



import argparse
import os
from pygendata import DataGenerator

parser = argparse.ArgumentParser(prog='pygendata', description='Python data generation library')

# Arguments
# generate: (csv, tsv, json)
# from: (ddl, tsv, json) <filepath>
# --rows (optional, default 100)
# to: (filepath to place results)
# pygendata --generate csv --base ddl /path/to/ddl --to /out/users.csv --rows 1000

parser.add_argument('--generate', type=str, help='The type of file to generate, options are csv, tsv, json')
parser.add_argument('--base', nargs=2, type=str, help='The type of file to generate data from, optons (ddl, json, tsv, csv) and the filepath where this file is located')
parser.add_argument('--template', nargs=2, help='The template to use to generate data (geo, medical, cellular ect..)')
parser.add_argument('--to', type=str, help='The filepath to place the results of the data generation in')
parser.add_argument('--rows', type=int, help='The number of rows to generate(default=100)')


def run():
    dg = None
    args = parser.parse_args()
    base_file_type = None
    base_file_loc = None
    template = None
    template_arg = None
    file_dest = args.to
    num_rows = args.rows
    filetype_to_generate = args.generate
    
    dg = DataGenerator(filetype_to_generate, rows=num_rows)
    if args.base:
        base_file_type, base_file_loc = args.base
    if args.template:
        template, template_arg = args.template
    
    filepath = os.getcwd()
    
    if base_file_type == 'ddl':
        dg.ddl(f"{filepath}/{base_file_loc}", f"{filepath}/{file_dest}") # this will need to be fixed when its used outside of this env
    elif template and template == 'geo':
        dg.template('geo', template_arg, f"{filepath}/{file_dest}")

if __name__ == "__main__":
    run()



import csv
import logging
from tqdm import tqdm

class CSVManager:
    def __init__(self, headers=None, rows=None):
        self.headers = headers
        self.rows = rows
    
    @property
    def headers(self):
        return self._headers

    @headers.setter
    def headers(self, headers):
        self._headers = headers

    @property
    def rows(self):
        return self._rows

    @rows.setter
    def rows(self, rows):
        self._rows = rows

    def read(self, file):
        try:
             with open(file, 'r') as f:
                statement  = f.read()
                return statement
        except IOError as e:
            logging.warning(str(e))
    
    def write(self, file):
        try:
            with open(file, 'w+') as csvfile:
                writer = csv.DictWriter(csvfile, fieldnames=self.headers)
                writer.writeheader()
                print('Writing rows to csv file')
                for row in tqdm(self.rows):
                    writer.writerow(row)
        except Exception as e:
            logging.warn(str(e))
    
    def __eq__(self, other):
        if not isinstance(other, CSVManager):
            return NotImplemented
        return self.headers == other.headers and self.rows == other.rows
            


import json
from json.decoder import JSONDecodeError
import logging
from tqdm import tqdm

class JSONManager:
    def __init__(self):
        self.headers = []
        self.rows = []
    
    @property
    def headers(self):
        return self._headers
    
    @headers.setter
    def headers(self, headers):
        self._headers = headers
    
    @property
    def rows(self):
        return self._rows
    
    @rows.setter
    def rows(self, rows):
        self._rows = rows
    
    def read(self, file):
        try:
            with open(file, 'r') as f:
                data = f.read()
                json_data =json.load(data)
                return json_data
        except JSONDecodeError as e:
            logging.warning(str(e))
    
    def write(self, file):
        try:
            write_data = {"rows": []}
            with open(file, 'w+') as f:
                # json will always write a key with an array as a value like so {"rows": [....]}
                print('Writing rows to JSON file')
                for row in tqdm(self.rows):
                    write_data['rows'].append(row)
                f.write(json.dumps(write_data))
        except IOError as e:
            logging.warning(str(e))
        